<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Frostbite3;

use MAM\Daemon\Core\Registry;
use MAM\Daemon\Core\Player as CorePlayer;

/**
 * Player Class for Frostbite
 *
 * This class contains all player relevant functions/options
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 * @Todo add language ignoreKick ignoreBan ignoreTempBan ignoreKill ignoreWarn playerWarnedPublicMsg playerKilledPublicMsg
 */
class Player extends CorePlayer {
    /**
     * pbGuid
     *
     * @var string
     */
    private $pbGuid = '';
    /**
     * squad
     *
     * @var $squad
     */
    private $squad;

    /**
     * Says something to a Player. Use {{br}} for new Line
     *
     * @param string $msg
     * @return void
     */
    public function say($msg) {
        $msg = str_replace("{{br}}", "\n", $msg);
        $msg = wordwrap($msg, 120 - strlen($this->mod->getCV("daemon", "pmprefix")), "\n", false);
        $split = explode("\n", $msg);
        foreach ($split as $value) {
            //  $value = str_replace(array('"', "'"), '`', $value);
            $value = $this->mod->getCV("daemon", "pmprefix") . $value;
            $this->rcon->rcon(array("admin.say", $value, "player", $this->name));
        }
    }

    /**
     * Yell something to a Player. Only one Line
     *
     * @param string $msg
     * @param int $duration Duration in seconds
     * @return void
     */
    public function yell($msg, $duration = 5) {
        $msg = str_replace("{{br}}", " ", $msg);
        $this->mod->triggerEvent("rconYell", $msg);
        $split = explode("\n", $msg);
        foreach ($split as $value) {
            $value = str_replace(array('"', "'"), '`', $value);
            $this->rcon->rcon(array("admin.yell", $value, $duration, "player", $this->name));
        }
    }

    /**
     * Resets the Team
     *
     * @return void
     */
    public function resetTeam() {
        $this->team = "none";
    }

    /**
     * Set this Player to a team
     *
     * @param int $team
     */
    public function setTeam($team) {
        if ($team != $this->team) {
            $oldteam = $this->team;
            $this->team = $team;
            $this->mod->triggerEvent("playerTeamChange", array($this->guid, $oldteam));
        }
    }

    /**
     * Returns the Team of a Player
     *
     * @return int
     */
    public function getTeam() {
        if ($this->team == "none") {
            $out = $this->rcon->rcon(array("listPlayers", "player", $this->getName()));
            if ($this->rcon->getStatus($out != true) || !isset($out[15])) {
                $this->logging->warning("Error fetching team from '" . $this->name . "' maybe left");
                return false;
            }
            $this->team = $out[15];
        }
        return $this->team;
    }

    /**
     * Resets the Squad
     *
     * @return void
     */
    public function resetSquad() {
        $this->squad = "none";
    }

    /**
     * Set this Player to a squad
     *
     * @param int $squad
     */
    public function setSquad($squad) {
        if ($squad != $this->squad) {
            $oldsquad = $this->squad;
            $this->squad = $squad;
            $this->mod->triggerEvent("playerTeamChange", array($this->guid, $oldsquad));
        }
    }

    /**
     * Returns the Squad of a Player
     *
     * @return int
     */
    public function getSquad() {
        if ($this->squad == "none") {
            $out = $this->rcon->rcon("listPlayers player " . $this->name);
            if ($this->rcon->getStatus($out != true)) {
                $this->logging->warning("Error fetching squad from '" . $this->name . "' maybe left");
                return false;
            }
            $this->squad = $out[17];
        }
        return $this->squad;
    }

    /**
     * Returns the IP of a Player
     * If ip can't get after 5 trys mod will use no ip
     *
     * @return String IP of the Player
     */
    public function getIP() {
        if ($this->ip == '') {
            return false;
        }
        return $this->ip;
    }

    /**
     * Set the IP of a Player
     *
     * @param string $ip
     * @return void
     */
    public function setIP($ip) {
        $this->ip = $ip;
    }

    /**
     * Returns the PbID of this Player. PBID = PID + 1
     *
     * @return string PbID
     */
    public function getPbid() {
        return $this->pid;
    }

    /**
     * Returns the ping of this player
     *
     * @return int
     */
    public function getPing() {
        return $this->rcon->rcon("player.ping $this->name")[2];
    }

    /**
     * Returns the PB Guid of this player
     *
     * @return string
     */
    public function getPBGuid() {
        return $this->pbGuid;
    }

    /**
     * Sets the PB Guid of this player
     *
     * @param string $pbguid
     * @return void
     */
    public function setPBGuid($pbguid) {
        $this->pbGuid = $pbguid;
    }

    /**
     * kills someone
     * @param bool $reason
     * @param bool $killer
     * @return bool
     */
    public function kill($reason = false, $killer = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.kill')) {
            if ($killer != false) {
                $this->players[$killer]->say($this->mod->getLngString("ignoreKill"));
            }
            return false;
        }
        if (!empty($reason)) {
            $this->mod->findReason($reason);
        } else {
            $reason = $this->mod->getCV("punishment", "defkillreason");
        }

        if ($this->rcon->rconKillPlayer($this)) {
            $this->say($reason);
            $this->mod->triggerEvent("playerKilled", array($this->guid, $reason, $killer));
            if ($this->mod->getCV("punishment", "announce")) {
                $this->say($this->mod->getLngString("playerKilledPublicMsg", array("<REASON>", "<PLAYER>", "<admin>"), array($reason, $this->getName(), ($killer) ? $killer : "MOD")));
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * Kick someone with PB else with game server kick
     *
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @return boolean
     */
    public function kick($reason = false, $kicker = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.kick')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreKick"));
            }
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("punishment", "defkickreason");
        }

        if ($this->mod->getCV("punishment", "punkbuster")) {
            if ($this->pbGuid !== '') {
                $this->rcon->rcon(array("punkBuster.pb_sv_command", "pb_sv_kick $this->name $reason"));
            } else {
                $this->logging->notice("Error kicking player '" . $this->name . "' with pb_sv_kick, using admin.kickPlayer, PID: " . $this->pid . ", GUID:" . $this->guid);
                $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
            }
        } else {
            $this->say($this->mod->getLngString("playerKickReasonNoPb", array("<REASON>"), array($reason)));
            $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
        }

        $kickerlog = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got kicked (by: $kickerlog), reason: $reason, PID: $this->pid, GUID: $this->guid");


        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defkickreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerKickedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defkickreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerKickedPublicMsgAuto", $search, $replace));
            }
        }
        //Create timed event to avoid mam lags due sleep
        $this->mod->triggerEvent("playerKicked", array($this->guid, $reason, $kicker));

        return true;
    }

    /**
     * Ban someone with PB else with game server ban
     *
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @return boolean
     */
    public function ban($reason = false, $kicker = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.ban')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreBan"));
            }
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("daemon", "defkickreason");
        }
        if ($this->mod->getCV("punishment", "punkbuster")) {
            if ($this->pbGuid !== '') {
                $this->rcon->rcon(array("punkBuster.pb_sv_command", "pb_sv_ban $this->pid $reason"));
                $type = 'pb';
            } else {
                $type = 'guid';
                $this->logging->notice("Error banning player '" . $this->pid . "' with pb_sv_ban, using banList, PID: " . $this->name . ", GUID:" . $this->guid);
                $this->rcon->rcon(array("banList.add", "guid", $this->guid, "perm", $reason));
                $this->rcon->rcon(array("banList.save"));
                $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
            }
        } else {
            $type = 'guid';
            $this->say($this->mod->getLngString("playerBanReasonNoPb", array("<REASON>"), array($reason)));
            $this->rcon->rcon(array("banList.add", "guid", $this->guid, "perm", $reason));
            $this->rcon->rcon(array("banList.save"));
            $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
        }
        $kicker_log = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got banned (by: $kicker_log), reason: $reason, PID: $this->pid, GUID: $this->guid");
        $banhelper = Registry::getInstance()->banhelper;
        $banhelper->writeBan($this->guid, $this->pbGuid, $this->name, $reason, 0, $type);
        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defbanreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerBannedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defbanreason"));
                $this->rcon->rconSay($this->mod->getLngString("playerBannedPublicMsgAuto", $search, $replace));
            }
        }

        $this->mod->triggerEvent("playerBanned", array($this->guid, $reason, $kicker));

        return true;
    }

    /**
     * Tempban someone with PB else with game server kick
     *
     * @param string|bool $reason
     * @param string|bool $kicker kickers guid
     * @param int $time ban time in minutes
     * @return boolean
     */
    public function tempBan($reason = false, $kicker = false, $time = 0) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.ignore.tempban')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString("ignoreTempBan"));
            }
            return false;
        } elseif (!$reason) {
            $reason = $this->mod->getCV("daemon", "defkickreason");
        }
        if (!$time || !is_numeric($time) || $time <= 0) {
            $time = $this->mod->getCV("daemon", "deftempbandur");
        } else {
            $time = (int) $time;
        }
        if ($this->mod->getCV("punishment", "punkbuster")) {
            if ($this->pbGuid !== '') {
                $this->rcon->rcon((array("punkBuster.pb_sv_command", "pb_sv_kick $this->name $time $reason")));
                $type = 'pb';
            } else {
                $type = 'guid';
                $this->logging->notice("Error banning player '" . $this->pid . "' with pb_sv_kick, using banList, PID: " . $this->name . ", GUID:" . $this->guid);
                $this->rcon->rcon(array("banList.add", "guid", $this->guid, "seconds", $time * 60, $reason));
                $this->rcon->rcon(array("banList.save"));
                $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
            }
        } else {
            $this->say($this->mod->getLngString("playerBanReasonNoPb", array("<REASON>"), array($reason)));
            $type = 'guid';
            $this->rcon->rcon(array("banList.add", "guid", $this->guid, "seconds", $time * 60, $reason));
            $this->rcon->rcon(array("banList.save"));
            $this->rcon->rcon(array("admin.kickPlayer", $this->name, $reason));
        }

        $kickerlog = ($kicker !== false) ? $this->players[$kicker]->getName() : "MOD";
        $this->logging->info("Player '$this->name' got temporarily banned for $time minutes (by: $kickerlog), reason: $reason, PID: $this->pid, GUID: $this->guid");
        $banhelper = Registry::getInstance()->banhelper;
        $banhelper->writeBan($this->guid, $this->pbGuid, $this->name, $reason, time() + $time * 60, $type);
        if ($this->mod->getCV("punishment", "announce")) {
            if ($kicker !== false) {
                $search = array("<PLAYER>", "<KICKER>", "<REASON>", "<TIME>");
                $replace = array($this->getName(), $this->players[$kicker]->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defbanreason"), ($time) ? $time : $this->mod->getCV("daemon", "deftempbandur"));
                $this->rcon->rconSay($this->mod->getLngString("playerTempBannedPublicMsg", $search, $replace));
            } else {
                $search = array("<PLAYER>", "<REASON>", "<TIME>");
                $replace = array($this->getName(), $reason ? $reason : $this->mod->getCV("daemon", "defbanreason"), ($time) ? $time : $this->mod->getCV("daemon", "deftempbandur"));
                $this->rcon->rconSay($this->mod->getLngString("playerTempBannedPublicMsgAuto", $search, $replace));
            }
        }

        $this->mod->triggerEvent("playerTempBanned", array($this->guid, $reason, $time ? $time : $this->mod->getCV("daemon", "deftempbandur"), $kicker));

        return true;
    }

    /**
     * Notice that the player killed someone
     *
     * @param string $type killtype like teamkill or kill
     * @param string $victim who get killed by the player
     * @param string $weapon the weapon name which the player killed
     * @param int $damage the damage what the player dealed
     * @param string $bodypart the body part
     */
    public function kills($type, $victim, $weapon, $damage, $bodypart) {
        $this->lastvictim = $victim;
        $this->lastweaponkill = $weapon;
        $this->lastkillwastk = false;

        if ($type == "kill") {
            $this->kills++;
            $this->damagegiven += $damage;
            if ($bodypart == "headshot") {
                $this->headshots++;
            }
            $this->mod->triggerEvent("playerKill", array($this->guid, $victim, $weapon, $damage, $bodypart));
        } elseif ($type == "teamkill") {
            $this->teamdamagegiven += $damage;
            $this->teamkills++;
            $this->lastkillwastk = true;
            $this->mod->triggerEvent("playerTeamKill", array($this->guid, $victim, $weapon, $damage, $bodypart));
        }
    }

    /**
     * Notice that the player dies and start events cause of his death
     *
     * @param string $type deathtype like teamkill selfkill
     * @param string $killer the killer who killed the player
     * @param string $weapon the weapon name which the player was killed
     * @param int $damage the damage dealed to the player
     * @param string $bodypart the body part
     */
    public function death($type, $killer, $weapon, $damage, $bodypart) {
        $this->lastkiller = $killer;
        $this->lastweapondeath = $weapon;
        $this->deaths ++;
        $this->damagetaken += $damage;
        $this->lastdeathwastk = false;

        if ($type == "teamkill") {
            $this->teamdeaths ++;
            $this->lastdeathwastk = true;
            $this->mod->triggerEvent("playerTeamDeath", array($this->guid, $killer, $weapon, $damage, $bodypart));
        } elseif ($type == "selfkill") {
            $this->selfkills ++;
            $this->lastkiller = $this->guid;
            $this->lastvictim = $this->guid;
            $this->lastweaponkill = $weapon;
            $this->lastkillwastk = false;
            $this->mod->triggerEvent("playerSelfKill", array($this->guid, $weapon, $damage, $bodypart));
        } else {
            $this->mod->triggerEvent("playerDeath", array($this->guid, $killer, $weapon, $damage, $bodypart));
        }
    }

    /**
     * Updates the Players PID, Name if it was changed and logs it
     *
     * @param int $pid new Player-ID
     * @param string $name new player name
     */
    public function update($pid, $name = '') {
        $name = trim($name);
        if ($name != '' && $name !== $this->name) {
            $this->logging->info("Player '$this->name' changed name to '$name', PID: $this->pid, GUID: $this->guid");
            $oldname = $this->name;
            $this->name = $name;
            $this->mod->triggerEvent("playerNameChange", array($this->guid, $oldname, $name));
        }
        if ($pid !== $this->pid && $this->pid !== 99) {
            $this->logging->info("Player '$this->name's PID was changed from $this->pid to $pid, GUID: $this->guid");
            $oldpid = $this->pid;
            $this->pid = $pid;
            $this->mod->triggerEvent("playerPIDChange", array($this->guid, $oldpid, $pid));
        } elseif ($pid !== $this->pid && $this->pid === 99) {
            $oldpid = $this->pid;
            $this->pid = $pid;
            $this->mod->triggerEvent("playerPIDChange", array($this->guid, $oldpid, $pid));
        }
    }
}
