<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at => www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at => Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http =>//www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Quake3\Cod5;

use MAM\Daemon\Core\Registry;

/**
 * Weapon Class for Call od Duty 5
 *
 * This class contains all weapons for Call od Duty 5
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Weapons {
    /**
     * Contains all weapons
     *
     * @var array
     */
    private $weapons;
    /**
     * Contains all extensions like silencer, red dot ...
     *
     * @var array
     */
    private $extensions;
    /**
     * Contains special stuff like suicide, death by world ...
     *
     * @var array
     */
    private $special;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;

    /**
     * Constructor
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->logging = $registry->logging;

        $this->getWeapons();
    }

    /**
     * Defines the Weapons
     */
    private function getWeapons() {
        $json = \MAM\Daemon\Libraries\Helper\Helper::parseJson(__DIR__ . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . "weapons.json");
        $this->weapons = $json["weapons"];
        $this->extensions = $json["extensions"];
        $this->special = $json["special"];
    }

    /**
     * Returns an array with detailed information about the weapon
     *
     * The array contains the weapon section, the weapon name and the short weapon name
     *
     * @todo Optimize it
     * @param array|string $index
     * @return array
     */
    public function getWeapon($index) {
        if (!$index) {
            return array("section" => 'special', "name" => $this->special["none"], "short" => 'none');
        }

        if (is_array($index) && $index[0] == "none") {
            $weaponindex = $index[0];
            $specialindex = $index[1];
            $specialindex_array = explode("_", $specialindex);
        } elseif (is_array($index)) {
            $weaponindex = $index[0];
            $specialindex = $index[1];
        } else {
            $weaponindex = $index;
            $specialindex = NULL;
        }

        $index_array = explode("_", $weaponindex);
        foreach ($this->weapons as $section => $values) {
            foreach ($values as $short => $long) {
                if (strtolower($short) == strtolower($index_array[0])) {
                    return array("section" => $section, "name" => $this->weapons[$section][$short], "short" => $short);
                }
            }
        }

        foreach ($this->extensions as $section => $short) {
            if (strtolower($short) == strtolower($specialindex[1])) {
                return array("section" => $section, "name" => $this->extensions[$section][$short], "short" => $short);
            }
        }

        foreach ($this->special as $short => $long) {
            if (strtolower($short) == strtolower($specialindex)) {
                return array("section" => "special", "name" => $this->special[$short], "short" => $short);
            }
        }

        $this->logging->warning("Weapon with index: $weaponindex not found");
    }
}
