<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license Creative Commons BY-NC-ND 4.0 <http://www.creativecommons.org/licenses/by-nc-nd/4.0/>
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\GeoIP;

use MAM\Daemon\Core\Utils;

/**
 * GeoIP Commands
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 * @version 1.0.0
 */
class Commands extends Utils {

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'geoip';
    }

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();
        $this->runPlugin();
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerCommand("geo", false, "geoCommand", $this);
    }

    /**
     * Splits the overgiven command in sub commands
     *
     * This functions checks the parameters and calls the fitting function for the overgiven parameter
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function geoCommand($guid, $parameters) {
        if (empty($parameters)) {
            $this->geoIPOwn($guid);
        } else {
            if ($this->permissions->hasPermissions($guid, 'geoip.geo.others')) {
                $this->geoIPOther($guid, $parameters);
            }
        }
    }

    /**
     * Returns the country of the executing player
     *
     * @param string $guid Player Guid
     */
    private function geoIPOwn($guid) {
        $geoip_data = $this->geoip->getLocationData($this->players[$guid]->getIP());

        if (isset($geoip_data['country']['iso_code'])) {
            $country_code = $geoip_data['country']['iso_code'];
        } else {
            $country_code = $this->mod->getLngString('unknown');
        }

        if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'english') {
            $country_name = $geoip_data['country']['names']['en'];
        } else if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'german') {
            $country_name = $geoip_data['country']['names']['en'];
        } else {
            $country_name = $this->mod->getLngString('unknown');
        }

        $name = $this->players[$guid]->getName();

        $search = array(
            '<NAME>',
            '<COUNTRY_NAME>',
            '<COUNTRY_CODE>'
        );

        $replace = array(
            $name,
            $country_name,
            $country_code
        );

        $this->rcon->rconSay($this->mod->getLngString('geoipOwn', $search, $replace));
    }

    /**
     * Returns the country of an other player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return false if player not found
     */
    private function geoIPOther($guid, $parameters) {

        $parameters = implode(" ", $parameters);

        $player = $this->mod->findPlayerGuid($parameters);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters)));
            return false;
        }

        $geoip_data = $this->geoip->getLocationData($this->players[$player]->getIP());

        if (isset($geoip_data['country']['iso_code'])) {
            $country_code = $geoip_data['country']['iso_code'];
        } else {
            $country_code = $this->mod->getLngString('unknown');
        }

        if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'english') {
            $country_name = $geoip_data['country']['names']['en'];
        } else if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'german') {
            $country_name = $geoip_data['country']['names']['en'];
        } else {
            $country_name = $this->mod->getLngString('unknown');
        }

        $name = $this->players[$player]->getName();

        $search = array(
            '<NAME>',
            '<COUNTRY_NAME>',
            '<COUNTRY_CODE>'
        );

        $replace = array(
            $name,
            $country_name,
            $country_code
        );

        $this->players[$guid]->say($this->mod->getLngString('geoipOther', $search, $replace));
    }
}
