<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 */

namespace MAM\Plugins\Antiteamkill;

use MAM\Daemon\Core\Utils;

/**
 * Automatically kicks players that make to many teamkills
 *
 *   PLUGIN: Antiteamkill
 * =============
 * Automatically kicks players that make to many teamkills
 * !forgive can be used to forgive a teamkill
 * !tki shows your current TKI
 * CONFIG
 * ------
 * [antiteamkill]
 * enabled = 1
 * min = -3
 * start = 0
 * warn = 3
 * kick = 4
 * teamkill = 1
 * kill = 0.1
 * mode = "tempban"
 * kickreason = "Made too many teamkills"
 * forgive = 1
 * forgivetime = 30
 * forgivepoints = 1
 * COMMANDS
 * -------
 * !forgive
 * !tki
 * NAMESPACE: Antiteamkill
 * Classes:
 * antiteamkill
 *
 * Vars:
 * $antiteamkill
 *
 * @author ManuAdminMod
 * @todo Move to downloadable content
 */
class Antiteamkill extends Utils {

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();

        //Init Players
        foreach (array_keys($this->players) as $guid) {
            $this->playerInit($guid);
        }
    }

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV("antiteamkill", "enabled", 0);
        $this->mod->setDefaultCV("antiteamkill", "min", -3);
        $this->mod->setDefaultCV("antiteamkill", "start", 0);
        $this->mod->setDefaultCV("antiteamkill", "warn", 3);
        $this->mod->setDefaultCV("antiteamkill", "kick", 4);
        $this->mod->setDefaultCV("antiteamkill", "teamkill", 1);
        $this->mod->setDefaultCV("antiteamkill", "kill", 0.1);
        $this->mod->setDefaultCV("antiteamkill", "mode", "tempban");
        $this->mod->setDefaultCV("antiteamkill", "kickreason", "Made too many teamkills");
        $this->mod->setDefaultCV("antiteamkill", "forgive", 0);
        $this->mod->setDefaultCV("antiteamkill", "forgivetime", 30);
        $this->mod->setDefaultCV("antiteamkill", "forgivepoints", 1);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent("playerJoined", "playerInit", $this);
        $this->mod->registerEvent("playerTeamKill", "triggerTeamKill", $this);
        $this->mod->registerEvent("playerKill", "triggerKill", $this);

        $this->mod->registerCommand("tki", false, "commandTKI", $this);
        $this->mod->registerCommand("forgive", false, "commandForgive", $this);
    }

    /**
     * Initiates a player
     *
     * @param String $guid Guid of executing player
     */
    public function playerInit($guid) {
        $this->players[$guid]->antiteamkiller_tki = $this->mod->getCV("antiteamkill", "start");
        $this->players[$guid]->antiteamkiller_lastteamkilled = 0;
        $this->players[$guid]->antiteamkiller_lastTKforgiven = false;
        $this->players[$guid]->antiteamkiller_lastTeamKiller = "";
    }

    /**
     * Trigger for Teamkills
     *
     * @param String $parameters Additional parameters
     * @return void
     */
    public function triggerTeamKill($parameters) {
        list($killer, $victim, $weapon) = $parameters;

        if (!$this->mod->getCV("antiteamkill", "enabled")) {
            return;
        }

        $add = $this->mod->getCV("antiteamkill", "teamkill");
        //Airstrike
        if ($weapon[0] == "artillery_mp" || $weapon[0] == "destructible_car") {
            $add /= 2;
        } elseif ($weapon[0] == "claymore_mp" || $weapon[0] == "cobra_20mm_mp" || $weapon[0] == "hind_20mm_mp") {
            $add = 0;
        }
        $this->players[$killer]->antiteamkiller_tki += $add;

        $this->players[$victim]->antiteamkiller_lastteamkilled = time();
        $this->players[$victim]->antiteamkiller_lastTKforgiven = false;
        $this->players[$victim]->antiteamkiller_lastTeamKiller = $killer;
        $this->players[$victim]->antiteamkiller_lastTKWeapon = $weapon[0];

        $this->logging->info("Antiteamkiller: Player '" . $this->players[$killer]->getName() . "' teamkilled '" . $this->players[$victim]->getName() . "', PID_A: " . $this->players[$killer]->getPID() . ", GUID_A: " . $this->players[$killer]->getGuid() . ", PID_V: " . $this->players[$victim]->getPID() . ", GUID_V: " . $this->players[$victim]->getGuid());

        $kicked = $this->warnAndKick($killer);

        if (!$kicked) {
            $this->sendForgive($victim, $killer);
        }
    }

    /**
     * Trigger for kills
     *
     * @param String $parameters Additional parameters
     * @return void
     */
    public function triggerKill($parameters) {
        list($killer, $victim) = $parameters;

        if (!$this->mod->getCV("antiteamkill", "enabled")) {
            return;
        }

        if ($this->players[$killer]->antiteamkiller_tki - $this->mod->getCV("antiteamkill", "kill") <= $this->mod->getCV("antiteamkill", "min")) {
            $this->players[$killer]->antiteamkiller_tki = $this->mod->getCV("antiteamkill", "min");
        } else {
            $this->players[$killer]->antiteamkiller_tki -= $this->mod->getCV("antiteamkill", "kill");
        }
    }

    /**
     * Warns a player and kicks him if he has to much warns
     *
     * @param String $killer Guid of the killer
     * @return boolean true if kicked
     */
    private function warnAndKick($killer) {

        $tki = $this->players[$killer]->antiteamkiller_tki;

        if ($tki >= $this->mod->getCV("antiteamkill", "kick")) {
            switch ($this->mod->getCV("antiteamkill", "mode")) {
                case "tempban":
                    $this->players[$killer]->tempBan($this->mod->getCV("antiteamkill", "kickreason"));
                    break;
                case "kick":
                    $this->players[$killer]->kick($this->mod->getCV("antiteamkill", "kickreason"));
                    break;
                case "ban":
                    $this->players[$killer]->ban($this->mod->getCV("antiteamkill", "kickreason"));
                    break;
                default:
                    break;
            }
            return true;
        }
        if ($tki >= $this->mod->getCV("antiteamkill", "warn")) {
            $this->players[$killer]->say($this->mod->getLngString("warnHighTKI"));
            $this->logging->info("Antiteamkiller: Player '" . "' has been warned for his high TKI ($tki), PID: " . $this->players[$killer]->getPID() . ", GUID: " . $this->players[$killer]->getGuid());
        }
        return false;
    }

    /**
     * Shows your Teamkillindex
     *
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function commandTKI($guid, $parameters) {
        $tki = $this->players[$guid]->antiteamkiller_tki;
        $this->players[$guid]->say($this->mod->getLngString("getTKI", array("<TKI>"), array($tki)));
    }

    /**
     * Sends a forgive
     *
     * @param String $victim Guid of victim
     * @param String $killer guid of killer
     * @return void
     */
    private function sendForgive($victim, $killer) {
        if (!$this->mod->getCV("antiteamkill", "forgive")) {
            return;
        }
        $name = $this->players[$killer]->getName();
        $time = $this->mod->getCV("antiteamkill", "forgivetime");
        $this->players[$victim]->say($this->mod->getLngString("forgiveMessage", array("<TIME>", "<NAME>"), array($time, $name)));
    }

    /**
     * Command for forgive
     *
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @return void
     */
    public function commandForgive($guid, $parameters) {
        if (!$this->mod->getCV("antiteamkill", "forgive") || !array_key_exists($this->players[$guid]->antiteamkiller_lastTeamKiller, $this->players) || $this->players[$guid]->antiteamkiller_lastTKforgiven || time() - $this->players[$guid]->antiteamkiller_lastteamkilled > $this->mod->getCV("antiteamkill", "forgivetime")) {
            return;
        }

        $this->players[$guid]->antiteamkiller_lastTKforgiven = true;
        $weapon = $this->players[$guid]->antiteamkiller_lastTKWeapon;

        $remove = $this->mod->getCV("antiteamkill", "forgivepoints");
        //Airstrike
        if ($weapon == "artillery_mp" || $weapon == "destructible_car") {
            $remove /= 2;
        } elseif ($weapon == "claymore_mp" || $weapon == "cobra_20mm_mp" || $weapon == "hind_20mm_mp") {
            $remove = 0;
        }
        $this->players[$this->players[$guid]->antiteamkiller_lastTeamKiller]->antiteamkiller_tki -= $remove;

        $name_killer = $this->players[$this->players[$guid]->antiteamkiller_lastTeamKiller]->getName();
        $name_victim = $this->players[$guid]->getName();

        $this->rcon->rconSay($this->mod->getLngString("playerHasForgiven", array("<KILLER>", "<VICTIM>"), array($name_killer, $name_victim)));

        $this->logging->info("Antiteamkill: Player '$name_victim' forgave is teamkill by '$name_killer', PID_V: " . $this->players[$guid]->getPID() . ", GUID_V: " . $guid . ", PID_A: " . $this->players[$this->players[$guid]->antiteamkiller_lastTeamKiller]->getPID() . ", PID_A: " . $this->players[$this->players[$guid]->antiteamkiller_lastTeamKiller]->getGuid());
    }
}
