<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Customcommands;

use MAM\Daemon\Core\Utils;

/**
 * Implements custom Commands for quake games
 *
 * @author ManuAdminMod
 */
class Customcommands extends Utils {
    /**
     * Array with all custom commands
     *
     * @var array
     */
    private $commands = array();

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->readConfig();
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->registerCommands();

        $this->mod->registerEvent('parseConfig', 'readConfig', $this);
    }

    /**
     * Reads in the config file
     */
    public function readConfig() {
        $this->commands = $this->mod->getCV('customcommands', 'commands');
    }

    /**
     * Register all Commands from Custom Command Section
     */
    public function registerCommands() {
        foreach (array_keys($this->commands) as $command) {
            $this->mod->registerCommand(strtolower($command), false, 'cmd_'.$command, $this);
        }
    }

    /**
     * Magic Method
     *
     * If Command exists, execute it, if not throw an error
     *
     * @param String $name Command Name
     * @param Array $parameters Command Parameters
     */
    public function __call($name, $parameters) {
        if (substr($name, 0, 4) != 'cmd_' || !array_key_exists(substr($name, 4), $this->commands)) {
            $this->logging->warning('Customcommands: Call to undefined function ' . $name);
        }

        $this->executeCommand(substr($name, 4), $parameters[0], $parameters[1]);
    }

    /**
     * Executes the Command
     *
     * @param   string      $command    The command name
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  boolean
     */
    private function executeCommand($command, $guid, $parameters) {
        $parameter = implode(' ', $parameters);

        $search = array();
        $replace = array();

        if (preg_match('#<ARG:FIND_PLAYER:(PBID|GUID|PID|NAME)>#', $this->commands[$command])) {
            $player = $this->mod->findPlayerGuid($parameter);
            if ($player === false) {
                $this->players[$guid]->say($this->mod->getLngString('playerNotFound', '<SEARCH>', $parameter));
                return;
            }

            $search[] = '<ARG:FIND_PLAYER:PID>';
            $search[] = '<ARG:FIND_PLAYER:GUID>';
            $search[] = '<ARG:FIND_PLAYER:PBID>';
            $search[] = '<ARG:FIND_PLAYER:NAME>';
            $replace[] = $this->players[$player]->getPID();
            $replace[] = $guid;
            $replace[] = $this->players[$player]->getPbid();
            $replace[] = $this->players[$player]->getName();
        }

        if (strpos($this->commands[$command], '<ARG:FIND_MAP>') !== false) {
            $map = $this->mod->findMap($parameter);
            if ($map == false) {
                $this->players[$guid]->say($this->mod->getLngString('mapNotFound', '<MAP>', $parameter));
                return;
            }

            $search[] = '<ARG:FIND_MAP>';
            $replace[] = $map;
        }

        $search[] = '<ARG>';
        $search[] = '<PLAYER:GUID>';
        $search[] = '<PLAYER:PID>';
        $search[] = '<PLAYER:PBID>';
        $search[] = '<PLAYER:NAME>';
        $search[] = '<PLAYER:ADMINGROUP_SHORT>';
        $search[] = '<PLAYER:ADMINGROUP_LONG>';


        $replace[] = $parameter;
        $replace[] = $guid;
        $replace[] = $this->players[$guid]->getPID();
        $replace[] = $this->players[$guid]->getPbid();
        $replace[] = $this->players[$guid]->getName();
        $replace[] = $this->players[$guid]->getGroup();
        $replace[] = $this->mod->getLongGroupName($this->players[$guid]->getGroup());

        if (preg_match('#<LAST_KILLER:(PBID|GUID|PID|NAME)>#', $this->commands[$command])) {
            $sguid = $this->players[$guid]->getLastKiller();
            if (!$sguid) {
                $this->players[$guid]->say($this->mod->getLngString('noKiller'));
                return;
            } else {
                $search[] = '<LAST_KILLER:GUID>';
                $search[] = '<LAST_KILLER:PID>';
                $search[] = '<LAST_KILLER:PBID>';
                $search[] = '<LAST_KILLER:NAME>';
                $replace[] = $sguid;
                $replace[] = $this->players[$sguid]->getPID();
                $replace[] = $this->players[$sguid]->getPbid();
                $replace[] = $this->players[$sguid]->getName();
            }
        }

        if (preg_match('#<LAST_VICTIM:(PBID|GUID|PID|NAME)>#', $this->commands[$command])) {
            $sguid = $this->players[$guid]->getLastVictim();
            if (!$sguid) {
                $this->players[$guid]->say($this->mod->getLngString('noVictim'));
                return;
            } else {
                $search[] = '<LAST_VICTIM:GUID>';
                $search[] = '<LAST_VICTIM:PID>';
                $search[] = '<LAST_VICTIM:PBID>';
                $search[] = '<LAST_VICTIM:NAME>';
                $replace[] = $sguid;
                $replace[] = $this->players[$sguid]->getPID();
                $replace[] = $this->players[$sguid]->getPbid();
                $replace[] = $this->players[$sguid]->getName();
            }
        }

        foreach (explode(';', $this->commands[$command]) as $command) {


            $rcon = str_replace($search, $replace, $command);

            if (!empty($parameter)) {
                $rcon = preg_replace('#<ARG:OPT:(.*?)>#', $parameter, $rcon);
            } else {
                $rcon = preg_replace('#<ARG:OPT:(.*?)>#', '$1', $rcon);
            }


            $this->rcon->rcon($rcon);
        }

        $this->players[$guid]->say($this->mod->getLngString('commandExecuted'));
    }
}
