<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Pingkick;

use MAM\Daemon\Core\Utils;

/**
 * Kicks players whos ping is to high.
 *
 * PLUGIN: Pingkicker
 * ================
 *
 * ATTENTION: This ofunction might not work correctly cause of a bug in the CoD
 * RCON response
 *
 * CONFIG:
 * -------
 * [pingkicker]
 * enabled = 1                         ;Enable plugin
 * maxping = 150                       ;Maximum ping that a player can have
 * checkinterval = 30                  ;Check pings every X secounds
 * warnstokick = 2                     ;Warns until kick
 * mode = 'tempban'                    ;kick/ban/tempban
 * kickreason = 'Your ping is to high' ;Reason of a kick
 *
 * NAMESPACE: pingkicker
 * Functions:
 * pingkicker_main
 * pingkicker_playerInit
 *
 * Vars:
 * $pingkicker_lastcheck
 *
 * @author ManuAdminMod
 */
class PingKick extends Utils {
    /**
     * Unixtimestamp of the last check
     *
     * @var int
     */
    private $pingkicker_lastcheck;

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV('pingkicker', 'enabled', 0);
        $this->mod->setDefaultCV('pingkicker', 'maxping', 150);
        $this->mod->setDefaultCV('pingkicker', 'checkinterval', 30);
        $this->mod->setDefaultCV('pingkicker', 'warnstokick', 2);
        $this->mod->setDefaultCV('pingkicker', 'mode', 'tempban');
        $this->mod->setDefaultCV('pingkicker', 'kickreason', 'Your ping is to high');

        $this->pingkicker_lastcheck = time();

        //Init Players because playerJoined isn't called at startup
        array_map(array($this, 'playerInit'), array_keys($this->players));
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('playerJoined', 'playerInit', $this);
        $this->mod->registerEvent('everyTime', 'pingkickerMain', $this);
        $this->mod->registerEvent('nextMap', 'reset', $this);  //Do not check while the clients are loading the map
        $this->mod->registerCommand('pingkick', '~^pingkick \S.*$~i', 'pingkick', $this);
    }

    /**
     * CHecks if ping is to high and kicks a player
     *
     * @return void
     */
    function pingkickerMain() {
        if (!$this->mod->getCV('pingkicker', 'enabled')) {
            return;
        }

        if (time() - $this->pingkicker_lastcheck < $this->mod->getCV('pingkicker', 'checkinterval')) {
            return;
        }

        $this->pingkicker_lastcheck = time();

        $playerlist = $this->rcon->rconPlayerList();

        foreach ($playerlist as $value) {
            if (!array_key_exists($value['guid'], $this->players)) {
                continue;
            }
            if ($value['ping'] <= $this->mod->getCV('pingkicker', 'maxping')) {
                $this->players[$value['guid']]->pingkicker_warns = 0;
            } else {

                $this->players[$value['guid']]->pingkicker_warns ++;

                if ($this->players[$value['guid']]->pingkicker_warns >= $this->mod->getCV('pingkicker', 'warnstokick')) {
                    switch ($this->mod->getCV('pingkicker', 'mode')) {
                        case 'tempban':
                            $this->players[$value['guid']]->tempBan($this->mod->getCV('pingkicker', 'kickreason'));
                            break;
                        case 'kick':
                            $this->players[$value['guid']]->kick($this->mod->getCV('pingkicker', 'kickreason'));
                            break;
                        case 'ban':
                            $this->players[$value['guid']]->ban($this->mod->getCV('pingkicker', 'kickreason'));
                            break;
                        default:
                            break;
                    }
                } else {
                    $this->players[$value['guid']]->say($this->mod->getLngString('warningPingToHigh'));
                    $this->logging->info('Player \'' . $this->players[$value['guid']]->getName() . '\' has been warned for his high ping (' . $value['ping'] . '), PID: ' . $this->players[$value['guid']]->getPID() . ', GUID:' . $value['guid']);
                }
            }
        }
    }

    /**
     * Kicks a player with a ping higher as X
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function pingKick($guid, $parameters) {
        $parameters = implode(' ', $parameters);

        $playerlist = $this->rcon->rconPlayerList();
        $i = 0;
        foreach ($playerlist as $value) {
            if (!array_key_exists($value['guid'], $this->players)) {
                continue;
            }
            if ($value['ping'] >= $parameters) {
                $this->players[$value['guid']]->kick($this->mod->getLngString('pingKickReason', '<PING>', $parameters));
                $i++;
            }
        }
        $this->players[$guid]->say($this->mod->getLngString('pingKickNotify', array('<PLAYERS>', '<PING>'), array($i, $parameters)));
    }

    /**
     * Inits a player
     *
     * @param String $guid Guid of executing player
     */
    function playerInit($guid) {
        $this->players[$guid]->pingkicker_warns = 0;
    }

    /**
     * Resets the check
     */
    function reset() {
        $this->pingkicker_lastcheck = time() + 30;
    }
}
