<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Spreemessages;

use MAM\Daemon\Core\Utils;

/**
 * Spreemessages
 *
 * The Spreemessages of the ManuAdminMod.
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class SpreeMessages extends Utils {
    /**
     * Firstblood
     *
     * Guid of the player who draw the first blood
     *
     * @var string Guid
     */
    private $firstblood = false;

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        foreach (array_keys($this->players) as $guid) {
            $this->playerInit($guid);
        }

        $this->mod->setDefaultCV('spreemessages_config', 'enabled', 0);
        $this->mod->setDefaultCV('spreemessages_config', 'firstblood', 0);
        $this->mod->setDefaultCV('spreemessages_config', 'killsprees', 0);
        $this->mod->setDefaultCV('spreemessages_config', 'deathsprees', 0);
        $this->mod->setDefaultCV('spreemessages_config', 'spreeends', 0);
        $this->mod->setDefaultCV('spreemessages_config', 'multikills', 0);

        $this->mod->setDefaultCV('spreemessages_messages', 'firstblood', '^7<PLAYER_NAME> ^1drew first blood!');
        $this->mod->setDefaultCV('spreemessages_messages', '5', '^7<PLAYER_NAME> ^1is on a ^3killing spree! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '10', '^7<PLAYER_NAME> ^1is on a ^3rampage! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '15', '^7<PLAYER_NAME> ^1is ^3dominating! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '20', '^7<PLAYER_NAME> ^1is ^3unstoppable! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '25', '^7<PLAYER_NAME> ^1is ^3godlike!! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '30', '^7<PLAYER_NAME> ^1is ^3wicked sick!!! ^1(^2<KILLS> ^1kills in a row)');
        $this->mod->setDefaultCV('spreemessages_messages', '35', '^7<PLAYER_NAME> ^1is ^3real Potter!!! ^1(Woohoo, ^2<KILLS> ^1kills in a row!)');
        $this->mod->setDefaultCV('spreemessages_messages', '-10', '^7<PLAYER_NAME> ^1seems to have a bad day ... (^2<DEATHS>^1 deaths without a ^2kill^1)');
        $this->mod->setDefaultCV('spreemessages_messages', '-15', '^7<PLAYER_NAME> ^1is on the best way to get ^3victim^1 of the day... (^2<DEATHS> ^1deaths w/o ^2kill^1)');
        $this->mod->setDefaultCV('spreemessages_messages', '-20', '^7<PLAYER_NAME> ^1really gets his ^3ass kicked!^1 Oh well, ^2<DEATHS> ^1deaths without a ^2kill^1...');
        $this->mod->setDefaultCV('spreemessages_messages', 'multikill', '^7!! ^1Multikill ^7> ^2<PLAYER_NAME>^7 < ^1Multikill^7 !!');
        $this->mod->setDefaultCV('spreemessages_messages', 'monsterkill', '^7!!!! ^1MONSTERKILL ^7>>> ^2<PLAYER_NAME>^7 <<< ^1MONSTER KILL ^7!!!!');
        $this->mod->setDefaultCV('spreemessages_messages', 'end_kill', '^2<PLAYER_NAME>^1\'s killing spree ended (^2<KILLS> ^1kills). He was killed 8by ^3<KILLER_NAME>^1!');
        $this->mod->setDefaultCV('spreemessages_messages', 'end_teamkill', '^2<PLAYER_NAME>^1\'s killing spree ended (^2<KILLS> ^1kills). He was killed by TEAMMATE ^3<KILLER_NAME>^1!');
        $this->mod->setDefaultCV('spreemessages_messages', 'end_selfkill', '^2<PLAYER_NAME>^1\'s killing spree ended (^2<KILLS> ^1kills). He killed ^3himself^1, what a pity!');
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('playerKill', 'triggerKill', $this);
        $this->mod->registerEvent('playerTeamKill', 'triggerTeamKill', $this);
        $this->mod->registerEvent('playerSelfKill', 'triggerSelfKill', $this);
        $this->mod->registerEvent('nextMap', 'triggerNextMap', $this);

        $this->mod->registerEvent('playerJoined', 'playerInit', $this);
    }

    /**
     * trigger a kill
     *
     * @param array $parameters
     * @return void
     */
    public function triggerKill($parameters) {
        list($killer, $victim, $weapon, $damage, $bodypart) = $parameters;

        if (!$this->mod->getCV('spreemessages_config', 'enabled')) {
            return;
        }

        //Firstblood
        $this->firstblood($killer);

        //Killsprees
        $this->players[$killer]->spreemessages_killspree ++;
        $this->killSpree($killer, $this->players[$killer]->spreemessages_killspree);

        //Spreeend
        $this->spreeEnd($victim, $killer, $this->players[$victim]->spreemessages_killspree, 'kill');
        $this->players[$victim]->spreemessages_killspree = 0;

        //Deathsspree
        $this->players[$killer]->spreemessages_deathspree = 0;
        $this->players[$victim]->spreemessages_deathspree ++;
        $this->deathSpree($victim, $this->players[$victim]->spreemessages_deathspree);

        //Multikills
        if (time() - $this->players[$killer]->spreemessages_lastkill < 3) {
            $this->players[$killer]->spreemessages_multikills ++;
            $this->multiKill($killer, $this->players[$killer]->spreemessages_multikills);
        } else {
            $this->players[$killer]->spreemessages_multikills = 1;
        }
        $this->players[$killer]->spreemessages_lastkill = time();
    }

    /**
     * trigger a teamkill
     *
     * @param array $parameters
     * @return void
     */
    public function triggerTeamKill($parameters) {
        list($killer, $victim, $weapon, $damage, $bodypart) = $parameters;

        if (!$this->mod->getCV('spreemessages_config', 'enabled')) {
            return;
        }

        //Spreeend
        $this->spreeEnd($victim, $killer, $this->players[$victim]->spreemessages_killspree, 'teamkill');
        $this->players[$victim]->spreemessages_killspree = 0;

        //Deathsspree
        $this->players[$victim]->spreemessages_deathspree ++;
        $this->deathSpree($victim, $this->players[$victim]->spreemessages_deathspree);
    }

    /**
     * trigger a selfkill
     *
     * @param array $parameters
     * @return void
     */
    public function triggerSelfKill($parameters) {
        list($killer, $weapon, $damage, $bodypart) = $parameters;

        if (!$this->mod->getCV('spreemessages_config', 'enabled')) {
            return;
        }

        //Spreeend
        $this->spreeEnd($killer, $killer, $this->players[$killer]->spreemessages_killspree, 'selfkill');
        $this->players[$killer]->spreemessages_killspree = 0;

        //Deathsspree
        $this->players[$killer]->spreemessages_deathspree ++;
        $this->deathSpree($killer, $this->players[$killer]->spreemessages_deathspree);
    }

    /**
     * trigger the nextmap
     */
    public function triggerNextMap() {
        $this->firstblood = false;
        foreach (array_keys($this->players) as $guid) {
            $this->players[$guid]->spreemessages_killspree = 0;
            $this->players[$guid]->spreemessages_deathspree = 0;
            $this->players[$guid]->spreemessages_multikills = 0;
            $this->players[$guid]->spreemessages_lastkill = 0;
        }
    }

    /**
     * the spreemessage trigger
     *
     * @param string $message
     * @param string $player
     * @param int $kills
     * @param int $deaths
     * @param string $killer
     */
    private function spreemessage($message, $player, $kills = 0, $deaths = 0, $killer = false) {
        $text = $this->mod->getCV('spreemessages_messages', $message);
        $search = array(
            '<PLAYER_NAME>',
            '<KILLS>',
            '<DEATHS>',
            '<KILLER_NAME>'
        );
        $replace = array(
            $this->players[$player]->getName(),
            $kills,
            $deaths,
            ($killer) ? $this->players[$killer]->getName() : '',
        );
        $this->rcon->rconSay(str_replace($search, $replace, $text));
    }

    /**
     * Firtblood message
     *
     * @param string $killer
     */
    private function firstblood($killer) {
        if ($this->mod->getCV('spreemessages_config', 'firstblood') && !$this->firstblood) {
            $this->spreemessage('firstblood', $killer);
        }
        $this->firstblood = true;
    }

    /**
     * killspree message
     *
     * @param string $killer
     * @param int $kills
     * @return void
     */
    private function killSpree($killer, $kills) {
        if (!$this->mod->getCV('spreemessages_config', 'killsprees')) {
            return;
        }

        if ($this->mod->existsCV('spreemessages_messages', $kills)) {
            $this->spreemessage($kills, $killer, $kills);
        }
    }

    /**
     * deathspree message
     *
     * @param string $victim
     * @param int $deaths
     * @return void
     */
    private function deathSpree($victim, $deaths) {
        if (!$this->mod->getCV('spreemessages_config', 'deathsprees')) {
            return;
        }

        if ($this->mod->existsCV('spreemessages_messages', '-$deaths')) {
            $this->spreemessage('-' . $deaths, $victim, 0, $deaths);
        }
    }

    /**
     * spreemessage end
     *
     * @param string $victim
     * @param string $killer
     * @param int $kills
     * @param string $type
     * @return void
     */
    private function spreeEnd($victim, $killer, $kills, $type) {
        if (!$this->mod->getCV('spreemessages_config', 'spreeends') || $kills < $this->mod->getCV('spreemessages_config', 'spreeendsfrom')) {
            return;
        }

        $this->spreemessage('end_' . $type, $victim, $kills, 0, $killer);
    }

    /**
     * multikill message
     *
     * @param string $killer
     * @param int $kills
     * @return void
     */
    private function multiKill($killer, $kills) {
        if (!$this->mod->getCV('spreemessages_config', 'multikills')) {
            return;
        }

        if ($kills == 3) {
            $this->spreemessage('multikill', $killer, $kills);
        } elseif ($kills == 5) {
            $this->spreemessage('monsterkill', $killer, $kills);
        }
    }

    /**
     * playerInit for players
     *
     * @param string $guid
     */
    public function playerInit($guid) {
        $this->players[$guid]->spreemessages_killspree = 0;
        $this->players[$guid]->spreemessages_deathspree = 0;
        $this->players[$guid]->spreemessages_multikills = 0;
        $this->players[$guid]->spreemessages_lastkill = 0;
    }
}
