<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Tcp;

use MAM\Daemon\Core\Utils;

/**
 * TCP Console
 *
 * TCP Class
 *
 * @author manu
 */
class Tcp extends Utils {
    /**
     * Socket
     *
     * @var string
     */
    private $socket = false;
    /**
     * connections
     *
     * @var array
     */
    private $connections = array();
    /**
     * Enabled
     *
     * @var int
     */
    private $enabled = false;
    /**
     * userDB
     *
     * @var array
     */
    private $userDB = array();


    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV("tcp_query", "enabled", 0);
        $this->mod->setDefaultCV("tcp_query", "ip", "0.0.0.0");
        $this->mod->setDefaultCV("tcp_query", "port", 1337);
        $this->mod->setDefaultCV("tcp_query", "logintimeout", 30);
        $this->mod->setDefaultCV("tcp_query", "defaulaction", "say");
        $this->mod->setDefaultCV("tcp_query", "sayprefix", "^3TCP-Admin (<ADMIN>): ^7");
        $this->mod->setDefaultCV("tcp_query", "admingroups", array("master", "admin"));
        $this->mod->setDefaultCV("tcp_query", "disallowedcommands", array());
        $this->mod->setDefaultCV("tcp_query", "maxconnections", 5);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->enabled = $this->mod->getCV("tcp_query", "enabled");

        if (!$this->enabled) {
            return;
        }

        $this->initSocket();
        $this->readUserDB();

        $this->mod->registerEvent("everyTime", "everyTime", $this);
        $this->mod->registerEvent("rconSay", "eventRconSay", $this);
        $this->mod->registerEvent("logAction", "eventLogAction", $this);
        $this->mod->registerEvent("nextMap", "eventNextMap", $this);
        $this->mod->registerEvent("parseConfig", "eventReadConfig", $this);
        $this->mod->registerEvent("playerJoined", "eventJoin", $this);

        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $registry->tcpquery = $this;
    }

    /**
     * Destructor
     */
    public function __destruct() {
        if (is_resource($this->socket)) {
            socket_close($this->socket);
            foreach (array_keys($this->connections) as $conection) {
                unset($this->connections[$conection]);
            }
        }
    }

    /**
     * This function is called after every config reloading
     */
    public function eventReadConfig() {
        if ($this->enabled && !$this->mod->getCV("tcp_query", "enabled")) {
            $this->writeAll("You have been disconnected: TCP-Interface has been disabled by admin" . PHP_EOL);

            foreach (array_keys($this->connections) as $conection) {
                unset($this->connections[$conection]);
            }
        } elseif (!$this->enabled && $this->mod->getCV("tcp_query", "enabled")) {
            $this->enabled = true;
            $this->initSocket();
            $this->readUserDB();
        } elseif ($this->enabled) {
            $this->readUserDB();
        }
    }

    /**
     * Reads in the userdb
     *
     * @return false on fail
     */
    private function readUserDB() {
        $this->userDB = array();
        $db = file($this->mod->getConfigDir() . "plugins/tcp_user.db");
        if ($db === false) {
            $this->logging->error("TCP-Query: Could not open user database (" . $this->mod->getConfigDir() . "plugins/tcp_user.db)");
            return;
        }
        foreach ($db as $lineno => $line) {
            //Comments
            $line = preg_replace('|;.*|', "", trim($line));

            if (empty($line)) {
                continue;
            } elseif (!preg_match('|^([a-z0-9_]+):([a-z0-9_]+):([a-f0-9]{40})$|i', $line, $subpatterns)) {
                $this->logging->warning("TCP-Query: Error: Syntax error in tcp_user.db on line " . ($lineno + 1));
            } else {
                list ($dummy, $user, $group, $pw) = $subpatterns;
                $group = strtolower($group);
                $pw = strtolower($pw);
                if (!array_key_exists($group, $this->mod->getGroups())) {
                    $this->logging->warning("TCP-Query: Error: Group '$group' does not exist in tcp_user.db on line " . ($lineno + 1));
                    continue;
                }
                if (array_key_exists($user, $this->userDB)) {
                    $this->logging->warning("TCP-Query: Error: User '$user' already exists in tcp_user.db on line " . ($lineno + 1));
                    continue;
                }
                $this->userDB[$user] = array("group" => $group, "password" => $pw);
            }
        }
    }

    /**
     * Check if Login is valid
     *
     * @param String $user Username
     * @param String $password Password
     * @param String $group Ingame group
     * @return boolean true or false
     */
    public function isValidLogin(&$user, $password, &$group) {
        $user = strtolower($user);
        //if (!array_key_exists($user, $this->userDB)) return false;

        foreach ($this->userDB as $username => $userdata) {
            if (strtolower($username) == strtolower($user)) {
                if ($userdata["password"] == sha1($password)) {
                    $user = $username;
                    $group = $this->userDB[$user]["group"];
                    return true;
                } else {
                    return false;
                }
            }
        }

        return false;
    }

    /**
     * This function is called everytime
     *
     * @return boolean false on fail
     */
    public function everyTime() {
        if (!$this->enabled) {
            return;
        }

        $this->checkForConnections();

        foreach (array_keys($this->connections) as $conection) {
            //Vorsorge gegen Fehlermeldungen wenn die Verbindungen während der Schleifenausführung gelöscht wurden
            if (!array_key_exists($conection, $this->connections)) {
                continue;
            }

            $this->connections[$conection]->everyTime();
            if ($this->connections[$conection]->closed) {
                unset($this->connections[$conection]);
            }
        }
    }

    /**
     * Inits the Socket
     *
     * @return boolean false on fail
     */
    private function initSocket() {
        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if (!$this->socket) {
            $this->socketError();
            return;
        }

        // BugFix for reuse the binding
        if (!socket_set_option($this->socket, SOL_SOCKET, SO_REUSEADDR, 1)) {
            $this->socketError($this->socket);
            return;
        }

        if (!@socket_bind($this->socket, $this->mod->getCV("tcp_query", "ip"), $this->mod->getCV("tcp_query", "port"))) {
            $this->socketError($this->socket);
            return;
        }
        if (!socket_listen($this->socket)) {
            $this->socketError($this->socket);
            return;
        }
        if (!socket_set_nonblock($this->socket)) {
            $this->socketError($this->socket);
            return;
        }
    }

    /**
     * Writes an warning message in the log
     *
     * @param socket $socket
     */
    private function socketError($socket = NULL) {
        $this->enabled = false;
        $err = socket_last_error($socket);
        $this->logging->warning("TCP-Query: Socket-Error: " . socket_strerror($err));
    }

    /**
     * Check for incoming connections
     *
     * @return boolean false on fail
     */
    private function checkForConnections() {
        $connection = @socket_accept($this->socket);
        if (!$connection) {
            return false;
        }
        if (count($this->connections) >= $this->mod->getCV("tcp_query", "maxconnections")) {
            socket_write($connection, chr(\MAM\Plugins\Tcp\TcpLevel::MT_CONSOLE) . chr(0) . "Maximum connections reached, you'll be disconnected" . PHP_EOL);
            socket_close($connection);
            return;
        }

        socket_set_nonblock($connection);
        $this->connections[] = new \MAM\Plugins\Tcp\TcpConnection($connection);
    }

    /**
     * Says a message to every connected client
     *
     * @param String $msg
     * @return boolean false on fail
     */
    public function eventRconSay($msg) {
        if (!$this->enabled) {
            return;
        }

        $this->writeAll($msg . PHP_EOL, \MAM\Plugins\Tcp\TcpLevel::MT_RCONSAY);
    }

    /**
     * event log action
     *
     * @param int $line
     * @return mixed
     */
    public function eventLogAction($line) {
        if (!$this->enabled) {
            return;
        }

        if ($line["action"] == "say" || $line["action"] == "sayteam") {
            $parts = $line["parsed"];
            $this->writeAll($parts[2] . ": " . $parts[3] . PHP_EOL, \MAM\Plugins\Tcp\TcpLevel::MT_CHAT);
        }
        return;
    }

    /**
     * This function is called on map change
     *
     * @param array $dvars Array with map infos
     * @return boolean false on fail
     */
    public function eventNextMap($dvars) {
        if (!$this->enabled) {
            return;
        }

        $this->writeAll("Next map / map restart: " . $this->mod->getLongMapName($dvars["mapname"]) . " (" . $this->mod->getLongGameType($dvars["g_gametype"]) . ")" . PHP_EOL);
    }

    /**
     * Writes a message to everyone
     *
     * @param String $msg The message
     * @param TcpLevel $type  where you want to show the message
     */
    private function writeAll($msg, $type = \MAM\Plugins\Tcp\TcpLevel::MT_CONSOLE) {
        foreach (array_keys($this->connections) as $connection) {
            if (!$this->connections[$connection]->loggedin()) {
                continue;
            }
            $this->connections[$connection]->write($msg, $type);
        }
    }

    /**
     * Kicks a TCP User
     *
     * @param int $id User ID
     * @return boolean true or false
     */
    public function kick($id) {
        if (!$this->enabled) {
            return;
        }

        if (array_key_exists($id, $this->connections)) {
            $this->connections[$id]->kick();
            return true;
        }
        return false;
    }

    /**
     * Returns a list of connections
     *
     * @return boolean/array false or array with all connections
     */
    public function getConnections() {
        if (!$this->enabled) {
            return false;
        }

        $return = array();
        foreach (array_keys($this->connections) as $connection) {
            $return[$connection] = array(
                "id" => $connection,
                "ip" => $this->connections[$connection]->getIP(),
                "port" => $this->connections[$connection]->getPort(),
                "user" => $this->connections[$connection]->getUser(),
            );
        }
        return $return;
    }

    /**
     * This function is called on every join
     *
     * @param String $guid Players Guid
     * @return boolean false on fail
     */
    public function eventJoin($guid) {
        if (!$this->enabled) {
            return;
        }
        $name = $this->players[$guid]->getName();

        $this->writeAll("Player joined: '$name'" . PHP_EOL, \MAM\Plugins\Tcp\TcpLevel::MT_INFO);
    }

    /**
     * Disconnects a player
     *
     * @param String $guid
     * @return void
     */
    public function eventQuit($guid) {
        if (!$this->enabled) {
            return;
        }
        $name = $this->players[$guid]->getName();

        $this->writeAll("Player quit: '$name'" . PHP_EOL, \MAM\Plugins\Tcp\TcpLevel::MT_INFO);
    }
}
