<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Voting;

/**
 * Plugin Voting
 *
 * This plugin implements the Vote ban.
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class VoteBan extends \MAM\Plugins\Voting\Vote {
    /**
     * Player has left the server
     * @var boolean
     */
    private $playerHasQuit = false;

    /**
     * Checks the parameters
     *
     * @todo Disable if admin is on
     * @throws \MAM\Plugins\Voting\VoteException
     */
    protected function parameterCheck() {
        if (empty($this->parameters[0])) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("voteSpecifyPlayer"));
        }
        $search = implode(" ", $this->parameters);
        $this->parameters = $this->mod->findPlayerGuid($search);
        if (!$this->parameters) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($search)));
        }

        if ($this->mod->getCV("voting", "disableifadminon") || count($this->mod->getOnlineAdmins()) > 0) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("voteDisableIfAdminOn"));
        }
    }

    /**
     * Start a vote
     */
    public function start() {
        $say = $this->mod->getLngString("voteStarted") . " " . $this->mod->getLngString("voteBan", "<PLAYER>", $this->players[$this->parameters]->getName()) . " " . $this->mod->getLngString("voteYesNo");
        $this->rcon->rconSay($say);
    }

    /**
     * Cancel a vote
     */
    public function cancel() {
        if ($this->playerHasQuit) {
            $say = $this->mod->getLngString("voteCanceled") . " " . $this->mod->getLngString("votePlayerLeft");
        } else {
            $say = $this->mod->getLngString("voteCanceled") . " " . $this->mod->getLngString("voteBan", "<PLAYER>", $this->players[$this->parameters]->getName());
        }
        $this->rcon->rconSay($say);
    }

    /**
     * Vote passed
     */
    public function pass() {
        $this->rcon->rconSay($this->mod->getLngString("votePassed") . " " . $this->mod->getLngString("voteKick", "<PLAYER>", $this->players[$this->parameters]->getName()) . " " . $this->mod->getLngString("voteYesNo3", array("<YES>", "<NO>"), array($this->yes, $this->no)));
        if ($this->mod->getCV("voting", "banduration") > 0) {
            $this->players[$this->parameters]->tempBan("Voteban passed", false, $this->mod->getCV("voting", "banduration"));
        } else {
            $this->players[$this->parameters]->ban("Voteban passed");
        }
    }

    /**
     * Vote failed
     */
    public function failed() {
        $say = $this->mod->getLngString("voteFailed") . " " . $this->mod->getLngString("voteBan", "<PLAYER>", $this->players[$this->parameters]->getName()) . " " . $this->mod->getLngString("voteYesNo3", array("<YES>", "<NO>"), array($this->yes, $this->no));
        $this->rcon->rconSay($say);
    }

    /**
     * Annouce the vote
     */
    public function announce() {
        $say = $this->mod->getLngString("voteInProgress") . " " . $this->mod->getLngString("voteBan", "<PLAYER>", $this->players[$this->parameters]->getName()) . " " . $this->mod->getLngString("voteYesNo2", array("<YES>", "<NO>"), array($this->yes, $this->no));
        $this->rcon->rconSay($say);
    }

    /**
     * Player left the server
     *
     * @param string $guid guid of player
     */
    public function playerQuit($guid) {
        if ($guid == $this->parameters) {
            $this->selfcanceled = true;
            $this->playerHasQuit = true;
        }
    }
}
