<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Weaponrestrictions;

use MAM\Daemon\Core\Utils;

/**
 * Plugin for restriced weapons
 *
 * @author ManuAdminMod
 */
class WeaponRestrictions extends Utils {

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();

        //Init Players because playerJoined isn't called at startup
        array_map(array($this, "weaponrestrictions_playerInit"), array_keys($this->players));
    }

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV("weaponrestrictions", "enabled", 0);
        $this->mod->setDefaultCV("weaponrestrictions", "weapons", array());
        $this->mod->setDefaultCV("weaponrestrictions", "warnstokick", 2);
        $this->mod->setDefaultCV("weaponrestrictions", "mode", "tempban");
        $this->mod->setDefaultCV("weaponrestrictions", "kickreason", "Used a restricted weapon: <WEAPON>");
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent("playerJoined", "weaponrestrictions_playerInit", $this);
        $this->mod->registerEvent("playerKill", "weaponrestrictions_main", $this);
        $this->mod->registerEvent("playerTeamKill", "weaponrestrictions_main", $this);
    }

    /**
     * Inits the Players
     *
     * @param string $guid Guid of the Player
     */
    function weaponrestrictions_playerInit($guid) {
        $this->players[$guid]->weaponrestrictions_warns = 0;
    }

    /**
     * Checks if forbidden weapon is used
     *
     * @param array $parameters
     * @return void
     */
    function weaponrestrictions_main($parameters) {
        list($killer_guid, $victim_guid, $weapon) = $parameters;


        if (!$this->mod->getCV("weaponrestrictions", "enabled")) {
            return;
        }

        if ($this->permissions->hasPermissions($killer_guid, 'weaponrestrictions.punishment.ignore')) {
            return false;
        }

        $restricted = false;
        $weapons = $this->mod->getCV("weaponrestrictions", "weapons");
        $weapons = array_map('strtolower', $weapons);

        if ($weapon[1] == "MOD_MELEE") {
            if (in_array("knife", $weapons)) {
                $restricted = true;
            } else {
                //Don't warn when player knifed while he had a restricted weapon
                return;
            }
        } else {
            $weapon_info = $this->mod->getWeaponInfo($weapon[0]);

            if (in_array("@" . strtolower($weapon_info["section"]), $weapons) || in_array(strtolower($weapon[0]), $weapons) || in_array($weapon_info['short'], $weapons)) {
                $restricted = true;
            }
        }

        if (!$restricted) {
            return;
        }

        $this->players[$killer_guid]->weaponrestrictions_warns++;

        $weaponlong = $weapon_info["name"];


        $score = $this->mod->getCV("weaponrestrictions", "score");
        $this->players[$killer_guid]->punishmentScoreAdd($score);
        $action = $this->registry->punishment->checkScore($this->players[$killer_guid]->getPunishmentScore(), $score);
        if ($action !== false) {
            if ($action == "warn") {
                $this->registry->punishment->punish($killer_guid, $action, $score, array("reason" => $this->mod->getLngString("usedRestrictedWeapon", array("<WEAPON>"), array($weaponlong)), "kicker" => false));
                $this->logging->info("Player '" . $this->players[$killer_guid]->getName() . "' has been warned for using restricted weapon: $weapon[0], PID: " . $this->players[$killer_guid]->getPID() . ", GUID: " . $killer_guid);
            } else {
                $this->registry->punishment->punish($killer_guid, $action, $score, array("reason" => str_replace("<WEAPON>", $weaponlong, $this->mod->getCV("weaponrestrictions", "kickreason")), "kicker" => false));
            }
        }
    }
}
